import createDomElement from '../common/create-dom-element';
import * as automationIds from '../common/automation-ids';


export class PackageTable {

  constructor (packages) {
    this.packageTable = createDomElement('div', { 'data-automation-id': automationIds.packageTable, 'class': 'pt-table', 'id': 'package-table' });

    if (packages.length === 0) {
      const noPackages = createDomElement('div', { 'data-automation-id': automationIds.noPackages, 'textContent': 'No Results', 'class': 'pt-no-packages' });
      this.packageTable.appendChild(noPackages);
    }

    for (let i = 0; i < packages.length; i++) {
      const packageInfo = packages[i];
      const packageRow = createDomElement('div', { 'data-automation-id': automationIds.row + '-' + i, 'class': 'pt-package-row'});
      const packageHeader = this.createPackageHeader(packageInfo);
      const liabilityRow = this.createLiabilityRow(packageInfo.liabilities);

      packageRow.appendChild(packageHeader);
      packageRow.appendChild(liabilityRow);
      this.packageTable.appendChild(packageRow);
    }
  }

  getDomTable () {
    return this.packageTable;
  }

  createPackageHeader (packageInfo) {
    const packageElement = createDomElement('div', { 'data-automation-id': automationIds.rowHeader, 'class': 'pt-package-row-header' });
    const innerContainer = createDomElement('div', { 'data-automation-id': automationIds.rowHeaderInner, 'class': 'pt-package-row-header-inner' });
    const packageName = createDomElement('div', { 'data-automation-id': automationIds.packageName, 'textContent': packageInfo.name, 'class': 'pt-package-name pt-package-name-passing' });
    const version = createDomElement('div', { 'data-automation-id': automationIds.packageVersion, 'textContent': 'Version: ' + packageInfo.version, 'class': 'pt-package-version' });

    innerContainer.appendChild(packageName);
    innerContainer.appendChild(version);
    packageElement.appendChild(innerContainer);

    return packageElement;
  }

  createLiabilityRow (liabilities) {
    const row = createDomElement('div', { 'data-automation-id': automationIds.liabilitiesRow, 'class': 'pt-liabilities-row' });
    const rowInnerContainer = createDomElement('div', { 'data-automation-id': automationIds.liabilitiesRowInner, 'class': 'pt-liabilities-row-inner' });

    for (let i = 0; i < liabilities.length; i++) {
      const rowElementProps = { 'data-automation-id': automationIds.rowElement + '-' + i, 'class': 'pt-liability' };
      rowElementProps.textContent = liabilities[i].name;
      const rowElement = createDomElement('div', rowElementProps);

      rowInnerContainer.appendChild(rowElement);
      row.appendChild(rowInnerContainer);
    }
    return row;
  }
}