import { sortByName, sortByLargestList } from './custom-sorts';
import { PackageTable } from './package-table/package-table';
import { createDropdown } from './components/dropdown';
import { createFilterBar } from './components/filter-bar';

function getPackagesWithLiabilities(stackProjects) {
  const liabilitiesMap = {};
  Object
    .keys(stackProjects)
    .forEach(pkgName => {
      liabilitiesMap[pkgName] = [];
      const pkgDependencies =
        Object.keys(Object.assign({}, stackProjects[pkgName].devDependencies, stackProjects[pkgName].dependencies));
      pkgDependencies.forEach(depName => {
        liabilitiesMap[depName] ? liabilitiesMap[depName].push(pkgName) : liabilitiesMap[depName] = [pkgName];
      });
    });
  return Object
    .keys(liabilitiesMap)
    .filter(pkg => !!stackProjects[pkg])
    .map(pkg => {
      return {
        name: pkg,
        version: stackProjects[pkg].version,
        liabilities: liabilitiesMap[pkg]
          .map(liab => ({name: liab, version: stackProjects[pkg].version}))
          .sort(sortByName)
      }
    })
    .sort(sortByLargestList);
}

export function getPackages(stack) {
  return getPackagesWithLiabilities(stack.projects);
}

export function getStackFromServer() {
  const GET_STACK = '/api/stack';
  return fetch(GET_STACK)
    .then(response => {
      return response.json();
    })
    .catch(err => {
      console.error('Error occured while getting stack from server: ', err);
      throw new Error('Error occured while getting stack from server: ', err);
    });
}

function getPackageDOMTable(packages) {
  const packageTable = new PackageTable(packages);
  return packageTable.getDomTable();
}

export function getLiabilitiesViewWithPackages(packages) {
  const liabilitiesViewContainer = document.getElementById('liabilities-view');

  function updateLiabilitiesViewWith(packages) {
    liabilitiesViewContainer.replaceChild(getPackageDOMTable(packages), document.getElementById('package-table'));
  }

  function filterPackages(value) {
    if (value === 'largest-list') {
      updateLiabilitiesViewWith(packages.sort(sortByLargestList));
    } else if (value === 'alphabetical') {
      updateLiabilitiesViewWith(packages.sort(sortByName));
    } else { // place-in-stack
      // To be implemented
    }
  }
  const orderOptions = ['Largest List', 'Alphabetical', 'Place in Stack'];
  const orderDropdown = createDropdown({options: orderOptions, id: 'dropdown-order', onChange: filterPackages});
  const filterBar = createFilterBar([orderDropdown]);

  liabilitiesViewContainer.appendChild(filterBar);
  liabilitiesViewContainer.appendChild(getPackageDOMTable(packages));
  return liabilitiesViewContainer;
}