const http = require('http');
const path = require('path');
const existsSync = require('fs').existsSync;
const homeDir = require('os').homedir;
const express = require('express');
const listenOnFreePort = require('listen-on-free-port');
const {fetchFromGithub} = require('./github-fetcher');


const customConfigPath = process.env.STACK_VISUALIZER_CONFIG_PATH;
const defaultConfigPath = path.join(homeDir(), 'config.js');

let config;

if (existsSync(customConfigPath)) {
  config = require(customConfigPath);
} else if (existsSync(defaultConfigPath)) {
  config = require(defaultConfigPath);
} else {
  throw new Error('Could not find server configuration files');
  exit(1);
}




function createServer(https) {
  const app = express();

  const server = http.createServer(app);
  const errorFile = require('../../test-kit/fixtures/errors.test.json');

  app.use('/', express.static(path.join(__dirname, '../../dist')));
  app.use('/assets', express.static(path.join(__dirname, '../../assets')));

  app.get('/', function (req, res) {
    res.sendFile(path.join(__dirname, 'static/index.html'));
  });

  app.all('/*', function (req, res, next) {
    res.header('Access-Control-Allow-Origin', '*');
    next();
  });

  app.get(config.apiStackFileUrl, fetchFromGithub({path: config.stackPath, token: config.token}, https));

  app.get(config.apiErrorFileUrl, function (req, res, next) {
    res.setHeader('Content-Type', 'application/json');
    res.send(JSON.stringify(errorFile));
  });

  return listenOnFreePort(3000, [], () => {
    return server;
  }).then(server => {
    console.log(`Listening at port ${server.address().port}`);
    return server;
  })
    .catch(err => {
      console.error('Failed to start server: ', err);
    });
}

module.exports = {createServer};