const chai = require('chai');
const chaiAsPromised = require('chai-as-promised');
const chaiDom = require('chai-dom');
const { util } = chai;

chai.use(chaiAsPromised);
chai.use(chaiDom);

chai.Assertion.addMethod('havePackage', function (packageName) {
  const obj = util.flag(this, 'object');
  const objName = obj.header;
  const expectedMessage = `expected ${objName} to be: ${packageName}`;
  const notExpectedMessage = `expected ${objName} to not be ${packageName}`;

  const assertion = objName === packageName;

  return this.assert(assertion, expectedMessage, notExpectedMessage, true, obj);
});

chai.Assertion.addMethod('haveLiabilities', function (liabilities) {
  const obj = util.flag(this, 'object');
  const objLiabilities = obj.liabilities;
  const expectedMessage = `expected ${objLiabilities} to be: ${liabilities}`;
  const notExpectedMessage = `expected ${objLiabilities} to not be ${liabilities}`;

  const assertion = objLiabilities.join(' ') === liabilities;

  return this.assert(assertion, expectedMessage, notExpectedMessage, true, obj);
});

chai.Assertion.addMethod('dropdownOptions', function (expectedOptions) {
  const obj = util.flag(this, 'object');
  const generatedOptions = Array.from(obj.childNodes).map(child => child.textContent);
  const expectedMessage = `expected ${generatedOptions} to be: ${expectedOptions}`;
  const notExpectedMessage = `expected ${generatedOptions} to not be ${expectedOptions}`;

  const assertion = generatedOptions.join(' ') === expectedOptions.join(' ');

  return this.assert(assertion, expectedMessage, notExpectedMessage, true, obj);
});

chai.Assertion.addMethod('liabilities', function (expectedLiabilities) {
  const obj = util.flag(this, 'object');
  const objLiabilities = obj.liabilities.map(liability => liability.name);

  chai.assert.deepEqual(objLiabilities, expectedLiabilities, `expected ${objLiabilities} to equal ${expectedLiabilities}`);
});

chai.Assertion.addProperty('haveSameYCoord', function () {
  const rowElementsYs = util.flag(this, 'object');

  const expectedMessage = `expected ${rowElementsYs} to be the same`;
  const notExpectedMessage = `expected ${rowElementsYs} to not be the same`;

  const assertion = rowElementsYs.every(elem => elem === rowElementsYs[0]);

  return this.assert(assertion, expectedMessage, notExpectedMessage, true, rowElementsYs);
});

module.exports = {
  chai
};
