const { chai } = require('../../test-kit/test-helpers');
const { expect } = chai;

import { getPackages } from '../../src/client/packages-helpers';
import { sortByName, sortByLargestList } from '../../src/client/custom-sorts';
import { sampleStack } from '../../test-kit/client-aux';

describe('Custom sorting tests', function () {
  it('should sort lexicographically', function () {
    const desserts = [{name: 'pancake'}, {name: 'creme brulle'}, {name: 'muffin'}, {name: 'cupcakes'}, {name: 'icecream'}];
    const expectedSortedDesserts = [{name: 'creme brulle'}, {name: 'cupcakes'}, {name: 'icecream'}, {name: 'muffin'}, {name: 'pancake'}];

    const sortedDesserts = desserts.sort(sortByName);

    expect(sortedDesserts).to.eql(expectedSortedDesserts);
  });

  it('should sort by largest list first', function () {
    const arrayOfLists = [{liabilities: ['1', '1']}, {liabilities: ['1', '1', '1']}, {liabilities: ['1']}];
    const expectedArrayOfLists = [{liabilities: ['1', '1', '1']}, {liabilities: ['1', '1']}, {liabilities: ['1']}];

    const sortedArrayOfLists = arrayOfLists.sort(sortByLargestList);

    expect(sortedArrayOfLists).to.eql(expectedArrayOfLists);
  });
});

describe('Liabilities View Data Test', function () {
  it('should generate liabilities', function () {
    const packages = getPackages(sampleStack);
    expect(packages[0]).to.have.liabilities(['test-p-1-1', 'test-p-1-2']);
    expect(packages[1]).to.have.liabilities(['test-p-1-2-1', 'test-p-1-2-2']);
    expect(packages[2]).to.have.liabilities([]);
    expect(packages[3]).to.have.liabilities([]);
    expect(packages[4]).to.have.liabilities([]);
  });
});
